/*

    File: Menu.h

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef LANDSCAPE_SHARED_MENU_H
#define LANDSCAPE_SHARED_MENU_H

//fixme - fix this all
#define	MENU_X_MAX	60
#define	MENU_Y_MAX	40

enum {
	MENUFUNC_LPRESS,
	MENUFUNC_RPRESS,
	MENUFUNC_LDOWN,
	MENUFUNC_RDOWN,
	MENUFUNC_COUNT,
};

#define MENU_TEXT_LEN	64

class menu_t {
public:

	//now, as I add these, I cannot put their methods in the .h file
	//the methods cannot be inline because they reference the 'engine' api,
	//which cannot be in the header
	//
	//for static linking this will link both the game and engine methods to the game code
	//but for dynamic linking, the engine will not be able to see the game's method implementations
	//therefore the engine will have to implement its own
	//
	//my suggestion to this problem:
	//either try as hard as can be to not use any menu methods engine-side 
	//	(only possible by not instanciating any menus whatsoever)
	//or, for these base methods, only provide direct callbacks through the associated api methods
	//implement this on both sides, engine and game, and things should go smooth

	void *operator new(size_t size);
	void operator delete(void *mem);	//for now use 'destroy' instead

	menu_t(float x, float y, float sizeX, float sizeY);
	~menu_t(void);

	menu_t *		find(const char *name);
	void			addChild(menu_t *child);
	void			unlinkFromParent(void);
	static menu_t *	getRoot(void);

	//and here's the rest of the story!

	//system
	bool			active;
	menu_t			*parent, *child, *prev, *next;

	//input
	menu_t			*selectedChild;

	//user
	float			pos[2];					//for quad geom / input area
	float			size[2];
	unsigned int	color;
	int				texture;				//bg texture

	float			childofs[2];			//mainly for scrolling
	int				topmost_priority;		//in the tradition windows way - with a thread-system twist.  - = lower, + = higher

	virtual bool	doMouseClick(int event, float x, float y) { return false; }
	virtual void	setSize(float sx, float sy) { size[0] = sx; size[1] = sy; }

	//still in the air
	char			text[MENU_TEXT_LEN];	//used for text displaying
	char			identifier[32];			//used to keep track of windows
};

#endif